@@ -177,26 +177,23 @@ gst_alsasink_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&alsasink_sink_factory));
 }
+
 static void
 gst_alsasink_class_init (GstAlsaSinkClass * klass)
 {
   GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseSinkClass *gstbasesink_class;
-  GstBaseAudioSinkClass *gstbaseaudiosink_class;
   GstAudioSinkClass *gstaudiosink_class;
 
   gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
   gstbasesink_class = (GstBaseSinkClass *) klass;
-  gstbaseaudiosink_class = (GstBaseAudioSinkClass *) klass;
   gstaudiosink_class = (GstAudioSinkClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_alsasink_finalise);
-  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_alsasink_get_property);
-  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_alsasink_set_property);
+  gobject_class->finalize = gst_alsasink_finalise;
+  gobject_class->get_property = gst_alsasink_get_property;
+  gobject_class->set_property = gst_alsasink_set_property;
 
   gstbasesink_class->get_caps = GST_DEBUG_FUNCPTR (gst_alsasink_getcaps);
 
@@ -882,8 +879,7 @@ gst_alsasink_write (GstAudioSink * asink, gpointer data, guint length)
     err = snd_pcm_wait (alsa->handle, (4 * alsa->period_time / 1000));
     if (err < 0) {
       GST_DEBUG_OBJECT (asink, "wait error, %d", err);
-    }
-    else {
+    } else {
       err = snd_pcm_writei (alsa->handle, ptr, cptr);
     }
 
