@@ -107,6 +107,7 @@ gst_ssa_parse_init (GstSsaParse * parse, GstSsaParseClass * klass)
 
   parse->ini = NULL;
   parse->framed = FALSE;
+  parse->send_tags = FALSE;
 }
 
 static void
@@ -115,7 +116,7 @@ gst_ssa_parse_class_init (GstSsaParseClass * klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  object_class->dispose = GST_DEBUG_FUNCPTR (gst_ssa_parse_dispose);
+  object_class->dispose = gst_ssa_parse_dispose;
 
   element_class->change_state = GST_DEBUG_FUNCPTR (gst_ssa_parse_change_state);
 }
@@ -152,6 +153,7 @@ gst_ssa_parse_setcaps (GstPad * sinkpad, GstCaps * caps)
   }
 
   parse->framed = TRUE;
+  parse->send_tags = TRUE;
 
   priv = (GstBuffer *) gst_value_get_mini_object (val);
   g_return_val_if_fail (priv != NULL, FALSE);
@@ -222,8 +224,20 @@ gst_ssa_parse_remove_override_codes (GstSsaParse * parse, gchar * txt)
   return removed_any;
 }
 
+/**
+ * gst_ssa_parse_push_line:
+ * @parse: caller element
+ * @txt: text to push
+ * @start: timestamp for the buffer
+ * @duration: duration for the buffer
+ *
+ * Parse the text in a buffer with the given properties and
+ * push it to the srcpad of the @parse element
+ *
+ * Returns: result of the push of the created buffer
+ */
 static GstFlowReturn
-gst_ssa_parse_parse_line (GstSsaParse * parse, gchar * txt,
+gst_ssa_parse_push_line (GstSsaParse * parse, gchar * txt,
     GstClockTime start, GstClockTime duration)
 {
   GstFlowReturn ret;
@@ -274,7 +288,8 @@ gst_ssa_parse_parse_line (GstSsaParse * parse, gchar * txt,
   ret = gst_pad_push (parse->srcpad, buf);
 
   if (ret != GST_FLOW_OK) {
-    GST_DEBUG_OBJECT (parse, "Push returned flow %s", gst_flow_get_name (ret));
+    GST_DEBUG_OBJECT (parse, "Push of text '%s' returned flow %s", txt,
+        gst_flow_get_name (ret));
   }
 
   return ret;
@@ -291,25 +306,32 @@ gst_ssa_parse_chain (GstPad * sinkpad, GstBuffer * buf)
   if (G_UNLIKELY (!parse->framed))
     goto not_framed;
 
-  /* make double-sure it's 0-terminated and all */
-  txt = g_strndup ((gchar *) GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
+  if (G_UNLIKELY (parse->send_tags)) {
+    GstTagList *tags;
 
-  if (txt == NULL)
-    return GST_FLOW_UNEXPECTED;
-
-  ts = GST_BUFFER_TIMESTAMP (buf);
-  ret = gst_ssa_parse_parse_line (parse, txt, ts, GST_BUFFER_DURATION (buf));
-
-  if (ret != GST_FLOW_OK) {
-    GST_WARNING_OBJECT (parse, "Failed to parse dialog line '%s'", txt);
-    if (GST_CLOCK_TIME_IS_VALID (ts)) {
-      /* just advance time without sending anything */
-      gst_pad_push_event (parse->srcpad,
-          gst_event_new_new_segment (TRUE, 1.0, GST_FORMAT_TIME, ts, -1, ts));
-      ret = GST_FLOW_OK;
-    }
+    tags = gst_tag_list_new ();
+    gst_tag_list_add (tags, GST_TAG_MERGE_APPEND, GST_TAG_SUBTITLE_CODEC,
+        "SubStation Alpha", NULL);
+    gst_element_found_tags_for_pad (GST_ELEMENT (parse), parse->srcpad, tags);
+    parse->send_tags = FALSE;
   }
 
+  /* make double-sure it's 0-terminated and all */
+  txt = g_strndup ((gchar *) GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
+  if (txt == NULL)
+    goto empty_text;
+
+  ts = GST_BUFFER_TIMESTAMP (buf);
+  ret = gst_ssa_parse_push_line (parse, txt, ts, GST_BUFFER_DURATION (buf));
+
+  if (ret != GST_FLOW_OK && GST_CLOCK_TIME_IS_VALID (ts)) {
+    /* just advance time without sending anything */
+    gst_pad_push_event (parse->srcpad,
+        gst_event_new_new_segment (TRUE, 1.0, GST_FORMAT_TIME, ts, -1, ts));
+    ret = GST_FLOW_OK;
+  }
+
+  gst_buffer_unref (buf);
   g_free (txt);
 
   return ret;
@@ -319,8 +341,16 @@ not_framed:
   {
     GST_ELEMENT_ERROR (parse, STREAM, FORMAT, (NULL),
         ("Only SSA subtitles embedded in containers are supported"));
+    gst_buffer_unref (buf);
     return GST_FLOW_NOT_NEGOTIATED;
   }
+empty_text:
+  {
+    GST_ELEMENT_WARNING (parse, STREAM, FORMAT, (NULL),
+        ("Received empty subtitle"));
+    gst_buffer_unref (buf);
+    return GST_FLOW_OK;
+  }
 }
 
 static GstStateChangeReturn
