@@ -1158,7 +1158,7 @@ gst_video_test_src_checkers8 (GstVideoTestSrc * v, guchar * dest, int w, int h)
     for (x = 0; x < w; x += 8) {
       int len = MIN (8, w - x);
 
-      if ((x ^ y) & (1 << 4)) {
+      if ((x ^ y) & (1 << 3)) {
         p->rgb_color = p->rgb_colors + COLOR_GREEN;
         p->yuv_color = p->yuv_colors + COLOR_GREEN;
         p->gray_color = p->gray_colors + COLOR_GREEN;
@@ -1431,6 +1431,95 @@ gst_video_test_src_circular (GstVideoTestSrc * v, unsigned char *dest,
   }
 }
 
+void
+gst_video_test_src_gamut (GstVideoTestSrc * v, guchar * dest, int w, int h)
+{
+  int x, y;
+  paintinfo pi = { NULL, };
+  paintinfo *p = &pi;
+  struct fourcc_list_struct *fourcc;
+  struct vts_color_struct_yuv yuv_primary;
+  struct vts_color_struct_yuv yuv_secondary;
+  struct vts_color_struct_rgb rgb_primary = { 0 };
+  struct vts_color_struct_rgb rgb_secondary = { 0 };
+  struct vts_color_struct_gray gray_primary = { 0 };
+  struct vts_color_struct_gray gray_secondary = { 0 };
+
+  p->rgb_colors = vts_colors_rgb;
+  if (v->color_spec == GST_VIDEO_TEST_SRC_BT601) {
+    p->yuv_colors = vts_colors_bt601_ycbcr_100;
+  } else {
+    p->yuv_colors = vts_colors_bt709_ycbcr_100;
+  }
+  p->gray_colors = vts_colors_gray_100;
+  p->width = w;
+  p->height = h;
+  fourcc = v->fourcc;
+  if (fourcc == NULL)
+    return;
+
+  fourcc->paint_setup (p, dest);
+  p->paint_hline = fourcc->paint_hline;
+
+  for (y = 0; y < h; y++) {
+    int region = (y * 4) / h;
+
+    switch (region) {
+      case 0:                  /* black */
+        yuv_primary = p->yuv_colors[COLOR_BLACK];
+        yuv_secondary = p->yuv_colors[COLOR_BLACK];
+        yuv_secondary.Y = 0;
+        rgb_primary = p->rgb_colors[COLOR_BLACK];
+        rgb_secondary = p->rgb_colors[COLOR_BLACK];
+        gray_primary = p->gray_colors[COLOR_BLACK];
+        gray_secondary = p->gray_colors[COLOR_BLACK];
+        break;
+      case 1:
+        yuv_primary = p->yuv_colors[COLOR_WHITE];
+        yuv_secondary = p->yuv_colors[COLOR_WHITE];
+        yuv_secondary.Y = 255;
+        rgb_primary = p->rgb_colors[COLOR_WHITE];
+        rgb_secondary = p->rgb_colors[COLOR_WHITE];
+        gray_primary = p->gray_colors[COLOR_WHITE];
+        gray_secondary = p->gray_colors[COLOR_WHITE];
+        break;
+      case 2:
+        yuv_primary = p->yuv_colors[COLOR_RED];
+        yuv_secondary = p->yuv_colors[COLOR_RED];
+        yuv_secondary.V = 255;
+        rgb_primary = p->rgb_colors[COLOR_RED];
+        rgb_secondary = p->rgb_colors[COLOR_RED];
+        gray_primary = p->gray_colors[COLOR_RED];
+        gray_secondary = p->gray_colors[COLOR_RED];
+        break;
+      case 3:
+        yuv_primary = p->yuv_colors[COLOR_BLUE];
+        yuv_secondary = p->yuv_colors[COLOR_BLUE];
+        yuv_secondary.U = 255;
+        rgb_primary = p->rgb_colors[COLOR_BLUE];
+        rgb_secondary = p->rgb_colors[COLOR_BLUE];
+        gray_primary = p->gray_colors[COLOR_BLUE];
+        gray_secondary = p->gray_colors[COLOR_BLUE];
+        break;
+    }
+
+    for (x = 0; x < w; x += 8) {
+      int len = MIN (8, w - x);
+
+      if ((x ^ y) & (1 << 4)) {
+        p->rgb_color = &rgb_primary;
+        p->yuv_color = &yuv_primary;
+        p->gray_color = &gray_primary;
+      } else {
+        p->rgb_color = &rgb_secondary;
+        p->yuv_color = &yuv_secondary;
+        p->gray_color = &gray_secondary;
+      }
+      p->paint_hline (p, x, y, len);
+    }
+  }
+}
+
 static void
 paint_setup_I420 (paintinfo * p, unsigned char *dest)
 {
@@ -1471,13 +1560,15 @@ static void
 paint_hline_I420 (paintinfo * p, int x, int y, int w)
 {
   int x1 = x / 2;
-  int x2 = (x + w) / 2;
+  int w1 = (x + w) / 2 - x1;
   int offset = y * p->ystride;
   int offset1 = (y / 2) * p->ustride;
 
+  if (x + w == p->width && p->width % 2 != 0)
+    w1++;
   oil_splat_u8_ns (p->yp + offset + x, &p->yuv_color->Y, w);
-  oil_splat_u8_ns (p->up + offset1 + x1, &p->yuv_color->U, x2 - x1);
-  oil_splat_u8_ns (p->vp + offset1 + x1, &p->yuv_color->V, x2 - x1);
+  oil_splat_u8_ns (p->up + offset1 + x1, &p->yuv_color->U, w1);
+  oil_splat_u8_ns (p->vp + offset1 + x1, &p->yuv_color->V, w1);
 }
 
 static void
@@ -1688,13 +1779,14 @@ static void
 paint_hline_YUY2 (paintinfo * p, int x, int y, int w)
 {
   int x1 = x / 2;
-  int x2 = (x + w) / 2;
-  int offset;
+  int w1 = (x + w) / 2 - x1;
+  int offset = y * p->ystride;
 
-  offset = y * p->ystride;
+  if (x + w == p->width && p->width % 2 != 0)
+    w1++;
   oil_splat_u8 (p->yp + offset + x * 2, 2, &p->yuv_color->Y, w);
-  oil_splat_u8 (p->up + offset + x1 * 4, 4, &p->yuv_color->U, x2 - x1);
-  oil_splat_u8 (p->vp + offset + x1 * 4, 4, &p->yuv_color->V, x2 - x1);
+  oil_splat_u8 (p->up + offset + x1 * 4, 4, &p->yuv_color->U, w1);
+  oil_splat_u8 (p->vp + offset + x1 * 4, 4, &p->yuv_color->V, w1);
 }
 
 static void
@@ -1725,9 +1817,9 @@ paint_setup_Y41B (paintinfo * p, unsigned char *dest)
   p->yp = dest;
   p->ystride = GST_ROUND_UP_4 (p->width);
   p->up = p->yp + p->ystride * p->height;
-  p->ustride = GST_ROUND_UP_8 (p->width) / 4;
+  p->ustride = GST_ROUND_UP_16 (p->width) / 4;
   p->vp = p->up + p->ustride * p->height;
-  p->vstride = GST_ROUND_UP_8 (p->width) / 4;
+  p->vstride = GST_ROUND_UP_16 (p->width) / 4;
   p->endptr = p->vp + p->vstride * p->height;
 }
 
@@ -1735,13 +1827,15 @@ static void
 paint_hline_Y41B (paintinfo * p, int x, int y, int w)
 {
   int x1 = x / 4;
-  int x2 = (x + w) / 4;
+  int w1 = (x + w) / 4 - x1;
   int offset = y * p->ystride;
   int offset1 = y * p->ustride;
 
+  if (x + w == p->width && p->width % 4 != 0)
+    w1++;
   oil_splat_u8_ns (p->yp + offset + x, &p->yuv_color->Y, w);
-  oil_splat_u8_ns (p->up + offset1 + x1, &p->yuv_color->U, x2 - x1);
-  oil_splat_u8_ns (p->vp + offset1 + x1, &p->yuv_color->V, x2 - x1);
+  oil_splat_u8_ns (p->up + offset1 + x1, &p->yuv_color->U, w1);
+  oil_splat_u8_ns (p->vp + offset1 + x1, &p->yuv_color->V, w1);
 }
 
 static void
@@ -1760,13 +1854,15 @@ static void
 paint_hline_Y42B (paintinfo * p, int x, int y, int w)
 {
   int x1 = x / 2;
-  int x2 = (x + w) / 2;
+  int w1 = (x + w) / 2 - x1;
   int offset = y * p->ystride;
   int offset1 = y * p->ustride;
 
+  if (x + w == p->width && p->width % 2 != 0)
+    w1++;
   oil_splat_u8_ns (p->yp + offset + x, &p->yuv_color->Y, w);
-  oil_splat_u8_ns (p->up + offset1 + x1, &p->yuv_color->U, x2 - x1);
-  oil_splat_u8_ns (p->vp + offset1 + x1, &p->yuv_color->V, x2 - x1);
+  oil_splat_u8_ns (p->up + offset1 + x1, &p->yuv_color->U, w1);
+  oil_splat_u8_ns (p->vp + offset1 + x1, &p->yuv_color->V, w1);
 }
 
 static void
@@ -1860,11 +1956,11 @@ paint_setup_YVU9 (paintinfo * p, unsigned char *dest)
 
   p->yp = dest;
   p->ystride = GST_ROUND_UP_4 (p->width);
-  p->vp = p->yp + p->ystride * GST_ROUND_UP_4 (p->height);
+  p->vp = p->yp + p->ystride * h;
   p->vstride = GST_ROUND_UP_4 (p->ystride / 4);
-  p->up = p->vp + p->vstride * GST_ROUND_UP_4 (h / 4);
+  p->up = p->vp + p->vstride * h / 4;
   p->ustride = GST_ROUND_UP_4 (p->ystride / 4);
-  p->endptr = p->up + p->ustride * GST_ROUND_UP_4 (h / 4);
+  p->endptr = p->up + p->ustride * h / 4;
 }
 
 static void
@@ -1877,22 +1973,24 @@ paint_setup_YUV9 (paintinfo * p, unsigned char *dest)
   p->ystride = GST_ROUND_UP_4 (p->width);
   p->up = p->yp + p->ystride * h;
   p->ustride = GST_ROUND_UP_4 (p->ystride / 4);
-  p->vp = p->up + p->ustride * GST_ROUND_UP_4 (h / 4);
+  p->vp = p->up + p->ustride * h / 4;
   p->vstride = GST_ROUND_UP_4 (p->ystride / 4);
-  p->endptr = p->vp + p->vstride * GST_ROUND_UP_4 (h / 4);
+  p->endptr = p->vp + p->vstride * h / 4;
 }
 
 static void
 paint_hline_YUV9 (paintinfo * p, int x, int y, int w)
 {
   int x1 = x / 4;
-  int x2 = (x + w) / 4;
+  int w1 = (x + w) / 4 - x1;
   int offset = y * p->ystride;
   int offset1 = (y / 4) * p->ustride;
 
+  if (x + w == p->width && p->width % 4 != 0)
+    w1++;
   oil_splat_u8_ns (p->yp + offset + x, &p->yuv_color->Y, w);
-  oil_splat_u8_ns (p->up + offset1 + x1, &p->yuv_color->U, x2 - x1);
-  oil_splat_u8_ns (p->vp + offset1 + x1, &p->yuv_color->V, x2 - x1);
+  oil_splat_u8_ns (p->up + offset1 + x1, &p->yuv_color->U, w1);
+  oil_splat_u8_ns (p->vp + offset1 + x1, &p->yuv_color->V, w1);
 }
 
 static void
