@@ -72,7 +72,7 @@ GST_START_TEST (test_pb_utils_post_missing_messages)
   bus = gst_element_get_bus (pipeline);
 
   /* first, test common assertion failure cases */
-  ASSERT_CRITICAL (msg = gst_missing_uri_source_message_new (NULL, "http"));
+  ASSERT_CRITICAL (msg = gst_missing_uri_source_message_new (NULL, "http"););
   ASSERT_CRITICAL (gst_missing_uri_source_message_new (pipeline, NULL));
 
   ASSERT_CRITICAL (gst_missing_uri_sink_message_new (NULL, "http"));
@@ -548,17 +548,19 @@ GST_START_TEST (test_pb_utils_install_plugins)
 {
   GstInstallPluginsContext *ctx;
   GstInstallPluginsReturn ret;
-  gchar *details[] = { (char *) "detail1", (char *) "detail2", NULL };
-  gchar *details_multi[] =
-      { (char *) "detail1", (char *) "detail1", (char *) "detail2", NULL };
+  gchar *details[] = { "detail1", "detail2", NULL };
+  gchar *details_multi[] = { "detail1", "detail1", "detail2", NULL };
 
   ctx = gst_install_plugins_context_new ();
 
-  ASSERT_CRITICAL (ret = gst_install_plugins_sync (NULL, ctx));
+  ASSERT_CRITICAL (ret = gst_install_plugins_sync (NULL, ctx);
+      );
   ASSERT_CRITICAL (ret =
-      gst_install_plugins_async (NULL, ctx, result_cb, (gpointer) & marker));
+      gst_install_plugins_async (NULL, ctx, result_cb, (gpointer) & marker);
+      );
   ASSERT_CRITICAL (ret =
-      gst_install_plugins_async (details, ctx, NULL, (gpointer) & marker));
+      gst_install_plugins_async (details, ctx, NULL, (gpointer) & marker);
+      );
 
   /* make sure the functions return the right error code if the helper does
    * not exist */
@@ -703,33 +705,6 @@ GST_START_TEST (test_pb_utils_installer_details)
 
 GST_END_TEST;
 
-GST_START_TEST (test_pb_utils_versions)
-{
-  gchar *s;
-  guint maj, min, mic, nano;
-
-  gst_plugins_base_version (NULL, NULL, NULL, NULL);
-  gst_plugins_base_version (&maj, &min, &mic, &nano);
-  fail_unless_equals_int (maj, GST_PLUGINS_BASE_VERSION_MAJOR);
-  fail_unless_equals_int (min, GST_PLUGINS_BASE_VERSION_MINOR);
-  fail_unless_equals_int (mic, GST_PLUGINS_BASE_VERSION_MICRO);
-  fail_unless_equals_int (nano, GST_PLUGINS_BASE_VERSION_NANO);
-
-  s = gst_plugins_base_version_string ();
-  if (GST_PLUGINS_BASE_VERSION_NANO == 0) {
-    fail_if (strstr (s, "GIT") || strstr (s, "git") || strstr (s, "prerel"));
-  }
-  if (GST_PLUGINS_BASE_VERSION_NANO == 1) {
-    fail_unless (strstr (s, "GIT") || strstr (s, "git"));
-  }
-  if (GST_PLUGINS_BASE_VERSION_NANO >= 2) {
-    fail_unless (strstr (s, "Prerelease") || strstr (s, "prerelease"));
-  }
-  g_free (s);
-}
-
-GST_END_TEST;
-
 static Suite *
 libgstpbutils_suite (void)
 {
@@ -743,7 +718,6 @@ libgstpbutils_suite (void)
   tcase_add_test (tc_chain, test_pb_utils_get_codec_description);
   tcase_add_test (tc_chain, test_pb_utils_install_plugins);
   tcase_add_test (tc_chain, test_pb_utils_installer_details);
-  tcase_add_test (tc_chain, test_pb_utils_versions);
   return s;
 }
 
